##################################################################################
##################################################################################
####################		 LIBRARIES      	  ########################

library(vegan)

##################################################################################
##################################################################################
####################	 DATA EXPLORATION      	  ################################

#This function explores the data previously to calculate N90

#af is the data with abundances of species in each sample
#gf in the data with the group assigned to each sample
#perc is the percentage of samples to be removed in each resampling
#perc2 is the maximum percentage of samples that can be repeated in the next resampling

Data_explore<-function(af, gf, perc, perc2) {

group<-as.character(unique(gf$Group))
i<-1
	for (g in group){

	message("Group: \"", i, "\"\n")
	
	dbg<- af[gf$Group == g, ]
	n<-nrow(dbg)
	
	sum<-summary(dbg)

	message("Name of the group: \"", g, "\"\n")
	message("Number of samples of the group: \"", n, "\"\n")

	n1<-(n*perc)%/%100 #number of samples removed in each jackknife
	if (n1==0) {n1<-1} else {n1<-n1}

	message("Number of samples removed in each jack-knife: \"", n1, "\"\n")

	rep1<-(perc2*n1)%/%100
	message("Maximum number of samples repeated: \"", rep1, "\"\n")

	i<-(i+1)
	}
}

##################################################################################
##################################################################################
####################	 SIMILARITY FUNCTION    	  ######################## 

#This function returns the similatity matrix for each grup of samples

#dbg is the abundance for each group of samples 
#contr is the similarity matrix obtained

Similarity<-function(dbg, contr) {

	####combinations within group
	take <- t(combn(1:nrow(dbg), 2))
	take

	md<-numeric(ncol(dbg))
	me<-numeric(ncol(dbg))

	contr <- matrix(nrow = nrow(take), ncol = ncol(dbg+2))
	for (j in 1:nrow(take)) {
		for (i in 1:ncol(dbg)) {	
    			md[i] <- 2*min(dbg[take[j , 1],i], dbg[take[j, 2],i])
    			me <- dbg[take[j, 1], ] + dbg[take[j, 2], ]
    			contr[j, ] <- 100*(md / sum(me))
			}
	
	}
	contr<-data.frame(contr)
	colnames(contr)<-c(names(dbg))
	return(contr)
}

##################################################################################
##################################################################################
##################	CUMULATIVE NUMBER OF SPECIES   ###########################

#This function calculate the value of N90 

#dbg is the abundance for each group of samples 
#contr is the similarity matrix obtained from Similarity function
#cutoff is the percentage of contributing to withim-group similarity
#n90 is the value of the N90 in each resampling

####cutoff to obtain all the species contributing to similarity is 99

N90<-function(contr, dbg, cutoff, n90){ 

	ndbg<-ncol(dbg)
	x<- colMeans(contr)
	df<-data.frame(x)
	x<-c(names(dbg))
	y<-df$x
	
	z1<-data.frame(Species=rep(NA, ndbg), Similarity=rep(NA, ndbg))

	z1$Species<-x
	z1$Similarity<-y

	z1$Similarity <- as.numeric(as.character(z1$Similarity))

	db<-z1[order(z1$Similarity,decreasing = TRUE),]
	b<-sum(db$Similarity)

	##calculus of contribution to mean similarity

	db$contribution<-((100*db$Similarity)/b)

	##calculus of accumulated mean similarity \% and cut according to cutoff
	
	d<-0
	for (i in 1:ncol(dbg)){
		d<-(db$contribution[i]+d)
		db$acum[i]<-d
	}

	table<-db[which(db$acum<cutoff & db$contribution>0),]
	n90in<-nrow(table)
	n90<-n90in+1
	return(n90)
}

##################################################################################
##################################################################################
####################	      SIMPER TABLE    	  ################################


#This function calculate the SIMPER table associated to N90

#dbg is the abundance for each group of samples 
#contr is the similarity matrix obtained from Similarity function
#cutoff is the percentage of contributing to withim-group similarity
#table is the SIMPER table obtained

####cutoff to obtain all the species contributing to similarity is 99

SIMPER_table<-function (contr, dbg, cutoff, table){
	ndbg<-ncol(dbg)
	x1<-colMeans(contr)
	x2<-apply(contr, 2, sd)
	df1<-data.frame(x1)
	df2<-data.frame(x2)
	x<-c(names(dbg))
	y1<-df1$x1
	y2<-df2$x2

	z2<-data.frame(Species=rep(NA, ndbg), Similarity=rep(NA, ndbg), SD=rep(NA, ndbg))

	z2$Species<-x
	z2$Similarity<-y1
	z2$SD<-y2

	##ordenation of species by similarity
	
	z2$Similarity <- as.numeric(as.character(z2$Similarity))

	db<-z2[order(z2$Similarity,decreasing = TRUE),]
	b<-sum(db$Similarity)

	##contribution to mean similarity

	db$contribution<-((100*db$Similarity)/b)

	##calculus of accumulated mean similarity \% and cut according to 90\%

	d<-0
	for (i in 1:ncol(dbg)){
		d<-(db$contribution[i]+d)
		db$acum[i]<-d
	}

	r<-db[which(db$acum<=cutoff & db$contribution>0),]
	
	Abundance0<-apply(dbg,2,mean)
	Ab_SD0<-apply(dbg,2,sd)
	Ab_SD10<-data.frame(Ab_SD0)
	Ab0<-data.frame(Abundance0)
	x0<-c(names(dbg))
	y0<-Ab0$Abundance0
	w0<-Ab_SD10$Ab_SD0	
	z0<-cbind(x0,y0,w0)
	rb0<-merge(z0, r, by.x = "x0", by.y = "Species")
	table<-rb0[order(rb0$Similarity,decreasing = TRUE),]
	names(table)<-c("Species", "Av.Abund", "SD.Abund", "Av.Si", "SD.Si", "Contr", "Cum")
	return(table)
}

##################################################################################
##################################################################################
####################	 N90 RESAMPLING FUNCTION        ##########################


#This function makes the resampling on N90

#af is the data with abundances of species in each sample
#gf in the data with the group assigned to each sample
#cutoff is the percentage of contributing to withim-group similarity
#perc is the percentage of samples to be removed in each resampling
#perc2 is the maximum percentage of samples that can be repeated in the next resampling
#jkmax is the maximum numbre of resamplings
#r5 is an object to save resampling values 
#my_list is a list with the following objects: N90_jackknifes,N90_mean_values and SIMPER_table

N90_resampling<-function(af, gf, cutoff, perc, perc2, jkmax, r5, my_list){

	if (jkmax>9999){
		warning("the maximum number of jackknifes must be lower than 10000")
	}

	group<-as.character(unique(gf$Group))

	SIMPERdf=data.frame()
	r5_df<-data.frame()
	r5_df_total<-data.frame()
	r6_df<-data.frame()
	r6_df_total<-data.frame()

	ext<-0

	#### cta is the accumulated contribution percentage or cutoff

	for (g in group){
		ext<-ext+1

		dbg<- af[gf$Group == g, ]
		message("group: \"", g, "\"\n")

		n<-nrow(dbg)
		contr<-Similarity(dbg)
	
		r<-SIMPER_table(contr, dbg, 101)
		rt<-cbind(g,r)
		colnames(rt)<-c("Group", "Species", "Av.Abund", "SD.Abund", "Av.Si", "SD.Si", "Contr", "Cum")

		rtdf<-data.frame(rt)

		SIMPERdf<-rbind(SIMPERdf, rtdf)

		### jack-knife for number of species and similarity in the group of samples

		take1 <- t(combn(1:n, 2))
		mf <- data.frame(take1)

		#n1 is the number of samples replaced in each jack-knife

		n1<-(n*perc)%/%100

		if (n1==0) {n1<-1} else {n1<-n1}

		max1<-jkmax*10
		a3<-0
		m1<-1

		rep1<-(perc2*n1)%/%100

		if (n1==1){
			if (jkmax<n) {
				Nsp<-numeric(jkmax)
				Sim<-numeric(jkmax)
				vec<-sample(1:n, jkmax, replace=FALSE)
				
				for (i in 1:jkmax){
					dbg2<-dbg[c(-vec[i]),]
					aux<-numeric(n-1)
					aux = which(mf[,'X1'] == vec[i] |mf[,'X2'] == vec[i])
					contraux<-contr[c(-aux),]
					Nsp[i]<-N90(contraux, dbg2, cutoff)
					Sim[i]<-(100-(100*mean(vegdist(dbg2, method='bray'))))
				}
			message ("Number of jack-cknifes: \"", jkmax, "\"\n")
			} else {
				Nsp<-numeric(n)
				Sim<-numeric(n)
				take <- t(combn(1:n, n1))
				for (i in 1:n) {
					k<-take[i, 1:n1]
					k<-data.frame(k)
					dbg2<-dbg[c(-take[i, 1:n1]),]
					aux<-matrix(nrow=n1, ncol=(n-1))
						for (j in 1:n1){ 
						aux[j,] = which(mf[,'X1'] == k[j,1] |mf[,'X2'] == k[j,1])
						}
				contraux<-contr[c(-aux),]
				contraux
				Nsp[i]<-N90(contraux, dbg2, cutoff)
				Sim[i]<-(100-(100*mean(vegdist(dbg2, method='bray'))))
				}
			message("Number of jack-cknifes: \"", n, "\"\n")			
			}
		} else {
			Nsp<-numeric(jkmax)
			Sim<-numeric(jkmax)
			matr<-matrix(ncol=n1, nrow=jkmax)	

			for(k in 1:max1){
				m<-m1
				if (m==(jkmax+1)) break
				samp<-sample(1:n, n1, replace=F)
				if (m==1){
					matr[m,]<-samp
					j2<-data.frame(samp)
					dbg2<-dbg[c(-samp),]
					aux<-matrix(nrow=n1, ncol=(n-1))
						for (l in 1:n1){ 
							aux[l,] = which(mf[,'X1'] == j2[l,1] 
							|mf[,'X2'] == j2[l,1])
						}
					contraux<-contr[c(-aux),]
					Nsp[m]<-N90(contraux, dbg2, cutoff)
					Sim[m]<-(100-(100*mean(vegdist(dbg2, method='bray'))))

					m1<-(m+1)
				} else { 
					matr2<-matrix(nrow=(m-1), ncol=n1)	
					if (m==2) {
						matr2a<-matr[1,]
						matr2<-as.matrix(t(matr2a))
					} else {
						matr2<- matr[c(1:(m-1)), c(1:n1)]
					}
					res <- matrix(0, nrow =(m-1) , ncol = n1)
					for (i in 1:(m-1)){
						for (j in 1:n1){
							c<-matr2[i,j]
							for (k2 in 1: n1){
								d<-samp[k2]
								if (d==c) {
									res[i,j]<-(res[i,j]+1)
								} else {res[i,j]<-(res[i,j]+0)}
							}
						}
					}
					res2<-apply(res, 1, sum)
						if (any(res2 > rep1)) {	
							m1<-m
							next
						} else {
							matr[m,]<-samp
							j2<-data.frame(samp)
							dbg2<-dbg[c(-samp),]
							aux<-matrix(nrow=n1, ncol=(n-1))
								for (l in 1:n1){ 
									aux[l,] = which(mf[,'X1'] == j2[l,1]
									 |mf[,'X2'] == j2[l,1])
								}
							contraux<-contr[c(-aux),]
							Nsp[m]<-N90(contraux, dbg2, cutoff)
							Sim[m]<-(100-(100*mean(vegdist(dbg2, method='bray'))))
							m1<-(m+1)
							Nsp<-Nsp[1:m]
							Sim<-Sim[1:m]
						}
				}
			}
		message("Number of jack-cknifes: \"", m-1, "\"\n")
		}

	### calculus of mean value and standar deviation of the number of species 
	### and the total similarity of the group

		r5_df<-cbind(g,Nsp,Sim)
		colnames(r5_df)<-c("Group", "n90_jackknife", "Sim_jackknife")
		r5_df_total<-rbind(r5_df_total,r5_df)

		Nsp_sd<-sd(Nsp)
		Nsp_mean<-mean(Nsp)
		Sim_sd<-sd(Sim)
		Sim_mean<-mean(Sim)

		r6_df<-cbind(g, Nsp_mean, Nsp_sd, Sim_mean, Sim_sd)
		colnames(r6_df)<-c( "Group", "Av.N90", "SD.N90", "Av.Sim", "SD.Sim")
		r6_df_total<-rbind(r6_df_total,r6_df)

	}

	my_list <- list(r5_df_total, r6_df_total, SIMPERdf)
	names(my_list) <- c('N90_jackknifes','N90_mean_values','SIMPER_table') 
	return(my_list) 
}

#################################################################################
#################################################################################
#Running functions and saving objects

af<-read.csv("Data_sp.csv",header=TRUE,sep=",",dec=".")
gf<-read.csv("Data_group.csv",header=TRUE,sep=",",dec=".")

Data_explore(af, gf, perc=10, perc2=70)
results<-N90_resampling(af, gf, cutoff=90, perc=10, perc2=70, jkmax=9999)

a<-results$N90_jackknifes
b<-results$N90_mean_values
c<-results$SIMPER_table

write.table(a,paste("N90_jackknifes.csv",sep=""),sep=",",dec=".",row.names=F)
write.table(b,paste("mean_values.csv",sep=""),sep=",",dec=".",row.names=F)
write.table(c,paste("SIMPER_table.csv",sep=""),sep=",",dec=".",row.names=F)

##
